from abc import ABC, abstractmethod


class Endpoint(object):
    def __init__(self, client):
        super(Endpoint, self).__init__()
        self.client = client

    @property
    @abstractmethod
    def _url(self) -> str:
        pass

    def _fetch(self, url: str, **kwargs: dict) -> dict:
        response = self.client.get(url, **kwargs)
        return response.json()

    @abstractmethod
    def __call__(self, *args: tuple, **kwargs: dict) -> dict:
        pass



from types import SimpleNamespace
from datetime import date

from ._base import Endpoint


class Info(Endpoint):
    _url = '/titulos-publicos/tesouro/%s/informacoes/'

    def __call__(self, code: str) -> dict:
        url = self._url % code.upper()
        return self._fetch(url)


class Cupons(Endpoint):
    _url = '/titulos-publicos/tesouro/%s/cupons/'

    def __call__(self, code: str) -> dict:
        url = self._url % code.upper()
        return self._fetch(url)['dados']


class Busca(Endpoint):
    _url = '/titulos-publicos/tesouro/'

    def __call__(self) -> dict:
        return self._fetch(self._url)['dados']


class PrecoAtual(Endpoint):
    _url = '/titulos-publicos/tesouro/%s/precos/historico/'

    def __call__(self, code: str) -> dict:
        url = self._url % code.upper()
        return self._fetch(url)


class PrecoHistorico(Endpoint):
    _url = '/titulos-publicos/tesouro/%s/precos/historico/'

    def __call__(self, code: str, start: date = None, end: date = None) -> dict:
        url = self._url % code.upper()
        params = {'dataInicio': start, 'dataFim': end}
        params = {k: v for k, v in params.items() if v is not None}
        return self._fetch(url)['dados']


def tesouro(client) -> SimpleNamespace:
    precos = SimpleNamespace(
        atual=PrecoAtual(client),
        historico=PrecoHistorico(client),
    )

    return SimpleNamespace(
        info=Info(client),
        cupons=Cupons(client),
        busca=Busca(client),
        precos=precos
    )



from types import SimpleNamespace

import httpx

from ._bolsa import bolsa
from ._tesouro import tesouro


class Fintz(object):
    def __init__(self, key: str = None):
        super(Fintz, self).__init__()
        self.key = key or '29kWWEfWHB6wjtsGYCLWn9PJ8seFzgtR2Ju9UvyH'

        self.client = httpx.Client(
            base_url='https://api.fintz.com.br',
            headers={'X-Api-Key': self.key},
            timeout=10  # seconds
        )

        self.bolsa = bolsa(self.client)
        self.titulos = SimpleNamespace(tesouro=tesouro(self.client))

    def __del__(self):
        self.client.close()



from types import SimpleNamespace
from datetime import date

from ._base import Endpoint


class Dres(Endpoint):
    _url = '/bolsa/b3/%s/dres/'

    def __call__(self, ticker: str, year: int, quarter: int) -> dict:
        url = self._url % ticker.upper()
        params = {'year': year, 'quarter': quarter}
        return self._fetch(url, params=params)


class Info(Endpoint):
    _url = '/bolsa/b3/%s/info/'

    def __call__(self, ticker: str) -> dict:
        url = self._url % ticker.upper()
        return self._fetch(url)


class Busca(Endpoint):
    _url = '/bolsa/b3/'

    def __call__(self, q: str) -> dict:
        params = {'q': q}
        return self._fetch(self._url, params=params)


class Eventos(Endpoint):
    _url = '/bolsa/b3/%s/eventos/'

    def __call__(self, ticker: str, start: date = None, end: date = None) -> dict:
        url = self._url % ticker.upper()
        params = {'dataInicio': start, 'dataFim': end}
        params = {k: v for k, v in params.items() if v is not None}
        return self._fetch(url, params=params)['dados']


class Cotacoes(Endpoint):
    _url = '/bolsa/b3/%s/cotacoes/'

    def __call__(self, ticker: str) -> dict:
        url = self._url % ticker.upper()
        return self._fetch(url)


class Historico(Endpoint):
    _url = '/bolsa/b3/%s/historico/'

    def __call__(
        self,
        ticker: str,
        start: date = None,
        end: date = None,
        page: int = 1,
        size: int = 10
    ) -> dict:
        url = self._url % ticker.upper()
        params = {'dataInicio': start, 'dataFim': end, 'pagina': page, 'tamanho': size}
        params = {k: v for k, v in params.items() if v is not None}
        return self._fetch(url, params=params)['dados']


class Proventos(Endpoint):
    _url = '/bolsa/b3/%s/proventos/'

    def __call__(self, ticker: str, start: date = None, end: date = None) -> dict:
        url = self._url % ticker.upper()
        params = {'dataInicio': start, 'dataFim': end}
        params = {k: v for k, v in params.items() if v is not None}
        return self._fetch(url, params=params)['dados']


def bolsa(client):
    return SimpleNamespace(
        dres=Dres(client),
        info=Info(client),
        busca=Busca(client),
        eventos=Eventos(client),
        cotacoes=Cotacoes(client),
        historico=Historico(client),
        proventos=Proventos(client)
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fintz']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.23.3,<0.24.0']

setup_kwargs = {
    'name': 'fintz',
    'version': '0.0.4',
    'description': 'Python client to interact with Fintz API',
    'long_description': "# Fintz SDK\n\nBiblioteca python para você interagir com a API da Fintz\n\n## Sobre\n\nA fintz é uma API feita para você não ter que se preocupar com dados do mercado\nfinanceiro. É fácil de usar e você pode começar a construir ferramentas e\nserviços rapidamente.\n\nPara mais informações, entre em contato conosco pelo nosso [site][1].\n\n## Instalar\n\nA instalação é super simples, basta usar `pip`:\n\n```\npip install fintz\n```\n\n## Como usar?\n\nEstamos sempre trabalhando para melhorar a usabilidade do SDK. Queremos deixar\nbem intuitivo para qualquer pessoa. Desde desenvolvedores com experiência até\npessoas com baixo conhecimento na área de programação.\n\nAqui vão alguns exemplos de métodos que temos implementados:\n\n```py\nfrom fintz import Fintz\n\nfintz = Fintz()  # usa uma chave padrão limitada\n\nfintz.bolsa.dres('PETR4', year=2020, quarter=3)\nfintz.bolsa.info('PETR4')\nfintz.bolsa.busca('PETR4')\nfintz.bolsa.eventos('PETR4')\nfintz.bolsa.cotacoes('PETR4')\nfintz.bolsa.historico('PETR4')\nfintz.bolsa.proventos('PETR4')\n\nfintz.titulos.tesouro.info('NTNBP20450515')\nfintz.titulos.tesouro.cupons('NTNB20450515')\nfintz.titulos.tesouro.busca()\nfintz.titulos.tesouro.precos.atual('NTNBP20450515')\nfintz.titulos.tesouro.precos.historico('NTNBP20450515')\n```\n\nSe você estiver recebendo erros do tipo `429 Too Many Requests`, significa que\na chave padrão está sendo muito utilizada e você deverá entrar em contato\nconosco para conseguir uma chave nova com limites maiores. Para entrar em\ncontato, use o formulário em nosso [site][1].\n\nQuando tiver a chave nova, basta usar o SDK da seguinte forma:\n\n```py\nfrom fintz import Fintz\n\nfintz = Fintz('SUA_CHAVE_AQUI')\n```\n\n[1]: https://fintz.com.br/\n",
    'author': 'João Vicente Meyer',
    'author_email': '1994meyer@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
